<?php /**
 * @file
 * Contains \Drupal\mollom\Controller\DefaultController.
 */

namespace Drupal\mollom\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormState;
use Drupal\Core\Render\MainContent\MainContentRendererInterface;
use Drupal\mollom\Element\Mollom;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * Default controller for the mollom module.
 */
class AjaxController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface|\Drupal\Core\Form\FormCacheInterface
   */
  protected $formBuilder;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The main content to AJAX Response renderer.
   *
   * @var \Drupal\Core\Render\MainContent\MainContentRendererInterface
   */
  protected $ajaxRenderer;

  /**
   * Constructs a FormAjaxController object.
   *
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   * @param \Drupal\Core\Render\MainContent\MainContentRendererInterface $ajax_renderer
   *   The main content to AJAX Response renderer.
   */
  public function __construct(FormBuilderInterface $form_builder, MainContentRendererInterface $ajax_renderer) {
    $this->formBuilder = $form_builder;
    $this->ajaxRenderer = $ajax_renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('form_builder'),
      $container->get('main_content_renderer.ajax')
    );
  }


  /**
   * AJAX callback to retrieve a CAPTCHA.
   *
   * @param $type
   *   The new CAPTCHA type to retrieve, e.g. 'image' or 'audio'.
   * @param $form_build_id
   *   The internal form build id of the form to update the CAPTCHA for.
   * @param $contentId
   *   (optional) The associated content ID in the form.
   *
   * @return
   *   A JSON array containing:
   *   - content: The HTML markup for the new CAPTCHA.
   *   - captchaId: The ID for the new CAPTCHA.
   *
   * @todo Add error handling.
   * @todo Convert to form-based AJAX if possible.
   */
  public function getNewCaptcha($type, $form_build_id, $contentId = NULL) {
    $request = \Drupal::request();
    if (!$request->isXmlHttpRequest()) {
      throw new AccessDeniedHttpException();
    }

    // Load $form_state from cache or create a dummy state.
    $form_state = new FormState();
    $form = $this->formBuilder->getCache($form_build_id, $form_state);
    if (!$form) {
      return;
    }
    $form_state->set('mollom_captcha_required', $type);
    $form['mollom']['captcha_required']['#value'] = $type;
    $form['mollom']['contentId']['#value'] = $contentId;
    Mollom::addMollomCaptcha($form['mollom']);

    $current_captcha_id = $form_state->getValue(array('mollom', 'response', 'captcha', 'id'));
    if (!empty($current_captcha_id)) {
      $form_state->set(array('mollom', 'response', 'captcha', 'id'), $form['mollom']['captchaId']['#value']);
    }
    $this->formBuilder->setCache($form_build_id, $form, $form_state);
    // Return new content and CAPTCHA ID via JSON.
    $data = array(
      'content' => $form['mollom']['captcha']['#field_prefix'],
      'captchaId' => $form['mollom']['captchaId']['#value'],
    );
    return new JsonResponse($data);
  }

  /**
   * @todo: Migrate to D8
   * Ajax callback for retrieving a form behavior analysis image.
   *
   * Outputs the JSON encoded tracking information received from Mollom.  This
   * will include keys of:
   *   - tracking_url: the URL to the tracking image
   *   - tracking_id: an ID to track for the image
   */
  public static function mollom_fba_js() {
    // Deny GET requests to make automated security audit tools not complain
    // about a JSON Hijacking possibility.
    // @see http://capec.mitre.org/data/definitions/111.html
    // @see http://haacked.com/archive/2009/06/24/json-hijacking.aspx
    if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
      header($_SERVER['SERVER_PROTOCOL'] . ' 405 Method Not Allowed');
      // A HTTP 405 response MUST specify allowed methods.
      header('Allow: POST');
      drupal_exit();
    }

    $mollom = \Drupal::service('mollom.client');
    $tracking = $mollom->getTrackingImage();
    drupal_json_output($tracking);
  }

  /**
   * @todo: Migrate to D8
   * Helper function to determine if "report to mollom" is available for the
   * current entity.
   *
   * @param $type
   *   The type of the entity to check.
   * @param $entity
   *   The entity to check.  This can be either the entity object or an id.
   * @return bool
   *   True if reporting is available and false if not available.
   */
  public static function flagAccess($type, $entity) {
    // make sure that Mollom is active and user is able to report.
    if (!_mollom_access('report to mollom')) {
      return FALSE;
    }
    // Find out if this entity bundle is protected.
    if (!is_object($entity)) {
      $entities = entity_load($type, array($entity));
      $entity = $entities[$entity];
    }
    list($id, $rid, $bundle) = entity_extract_ids($type, $entity);
    return mollom_flag_entity_type_access($type, array($bundle));
  }

  /**
   * @todo: Migrate to D8
   * Callback handler for public users to report content as inappropriate.
   * This is step one of the two-step process.  The user can now indicate the
   * reason for the report as one of spam, quality, profanity, or unwelcome.
   *
   * @param $type
   *   The request type submitted, one of "ajax" or "nojs".
   * @param $entity
   *   The type of entity that is being reported.
   * @param $id
   *   The entity identifier that is being reported.
   * @param $source
   *   The optional internal source to be submitted along with feedback.
   */
  public static function flag($type, $entity, $id, $source = NULL) {
    $detail = FALSE;
    if ($type === 'nojs' || $_SERVER['REQUEST_METHOD'] !== 'POST') {
      $detail = TRUE;
    }
    $form = drupal_get_form('mollom_flag_reason_form', $entity, $id, $detail, $source);

    // If not submitted via Ajax post, then return a plain Drupal form page.
    if ($detail) {
      return $form;
    }

    $dialog = mollom_flag_dialog_check();
    if (isset($dialog['display form callback'])) {
      $function = $dialog['display form callback'];
      return $function($form, t('Report'));
    }

    // Deliver via custom Mollom dialog.
    $commands = array();
    $formHtml = '<div class="mollom-flag-container" role="dialog" aria-label="' . t('Report') . '">' . render($form) . '</div>';
    $commands[] = ajax_command_prepend(".mollom-flag-content-$entity:has(#mollom_$entity$id)",$formHtml);
    $page = array('#type' => 'ajax', '#commands' => $commands);
    ajax_deliver($page);
  }

}
