<?php

/**
 * @file
 * Contains Drupal\mollom\Tests\MollomTestBase.
 */

namespace Drupal\mollom\Tests;


use Drupal\Component\Utility\SafeMarkup;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\mollom\Utility\Logger;
use Drupal\mollom\Utility\MollomUtilities;
use Drupal\simpletest\WebTestBase;


abstract class MollomTestBase extends WebTestBase {

  /**
   * Modules to enable.
   *
   * @todo: Add mollom_test_server once added.
   *
   * @var array
   */
  public static $modules = ['dblog', 'mollom', 'node', 'comment', 'mollom_test_server'];

  /**
   * The text the user should see when they are blocked from submitting a form
   * because the Mollom servers are unreachable.
   */
  const FALLBACK_MESSAGE = 'The spam filter installed on this site is currently unavailable. Per site policy, we are unable to accept new submissions until that problem is resolved. Please try resubmitting the form in a couple of minutes.';

  /**
   * The text the user should see if there submission was determined to be spam.
   */
  const SPAM_MESSAGE = 'Your submission has triggered the spam filter and will not be accepted.';

  /**
   * The text the user should see if they did not fill out the CAPTCHA correctly.
   */
  const INCORRECT_MESSAGE = 'The word verification was not completed correctly. Please complete this new word verification and try again.';

  /**
   * The text the user should see if the textual analysis was unsure about the
   * content.
   */
  const UNSURE_MESSAGE = "To complete this form, please complete the word verification.";

  /**
   * The text the user should see if the textual analysis determined that there
   * was profane content.
   */
  const PROFANITY_MESSAGE = "Your submission has triggered the profanity filter and will not be accepted until the inappropriate language is removed.";


  /**
   * Indicates if the default setup permissions and modules should be
   * skipped.
   *
   * @var bool
   */
  public $disableDefaultSetup = FALSE;

  /**
   * An user with permissions to administer Mollom.
   *
   * @var \Drupal\user\UserInterface
   */
  public $adminUser;

  /**
   * Tracks Mollom messages across tests.
   *
   * @var array
   */
  protected $messages = array();

  /**
   * The public key used during testing.
   */
  protected $publicKey;

  /**
   * The private key used during testing.
   */
  protected $privateKey;

  /**
   * Flag indicating whether to automatically create testing API keys.
   *
   * If testing_mode is enabled, Mollom module automatically uses the
   * MollomDrupalTest client implementation. This implementation automatically
   * creates testing API keys when being instantiated (and ensures to re-create
   * testing API keys in case they vanish). The behavior is executed by default,
   * but depends on the 'mollom_testing_create_keys' variable being TRUE.
   *
   * Some functional test cases verify that expected errors are displayed in
   * case no or invalid API keys are configured. For such test cases, set this
   * flag to FALSE to skip the automatic creation of testing keys.
   *
   * @see MollomDrupalTest::$createKeys
   * @see MollomDrupalTest::createKeys()
   */
  protected $createKeys = TRUE;

  /**
   * Indicates if the test local server should be used in place of the
   * Mollom API.
   */
  protected $useLocal = FALSE;

  /**
   * Tracks Mollom session response IDs.
   *
   * @var array
   */
  protected $responseIds = array();

  /**
   * Mollom configuration settings.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $mollomSettings;

  /**
   * An instance of the Drupal Mollom client.
   *
   * @var \Drupal\mollom\API\DrupalClientInterface
   */
  protected $mollom;

  /**
   * The current log level to use on the site after testing.
   */
  protected $originalLogLevel;

  /**
   * {@inheritDoc}
   */
  protected function setUp() {
    $this->resetResponseID();
    $this->messages = array();

    if ($this->disableDefaultSetup) {
      self::$modules = array_diff(self::$modules, array('mollom'));
    }

    parent::setUp();
    $settings = \Drupal::configFactory()->getEditable('mollom.settings');
    $test_mode = $settings->get('test_mode');

    // Set the spun up instance to use the local test class.
    if ($this->useLocal) {
      $test_mode['use_local'] = TRUE;
    }

    // Omit warnings and create keys only when the test asked for it.
    $test_mode['create_keys'] = $this->createKeys;
    $test_mode['omit_warning'] = TRUE;
    $settings->set('test_mode', $test_mode);

    // Set log level
    $settings->set('log_level', RfcLogLevel::DEBUG);
    $settings->save();

    if ($this->disableDefaultSetup) {
      return;
    }

    $permissions = array(
      'access administration pages',
      'administer mollom',
      'administer content types',
      'administer permissions',
      'administer users',
      'bypass node access',
      'access comments',
      'post comments',
      'skip comment approval',
      'administer comments',
    );
    $this->adminUser = $this->drupalCreateUser($permissions);

    if ($this->createKeys) {
      $this->setKeys();
      $this->assertValidKeys();
    }
  }

  /**
   * {@inheritDoc}
   */
  protected function tearDown() {
    $this->assertMollomWatchdogMessages();

    parent::tearDown();
  }

  /**
   * Assert any watchdog messages based on their severity.
   *
   * This function can be (repeatedly) invoked to assert new watchdog messages.
   * All watchdog messages with a higher severity than RfcLogLevel::NOTICE are
   * considered as "severe".
   *
   * @param $max_severity
   *   (optional) A maximum watchdog severity level message constant that log
   *   messages must have to pass the assertion. All messages with a higher
   *   severity will fail. Defaults to RfcLogLevel::NOTICE. If a severity level
   *   higher than RfcLogLevel::NOTICE is passed, then at least one severe message
   *   is expected.
   */
  protected function assertMollomWatchdogMessages($max_severity = RfcLogLevel::NOTICE) {
    // Ensure that all messages have been written before attempting to verify
    // them. Actions executed within the test class may lead to log messages,
    // but those get only logged when hook_exit() is triggered.
    // mollom.module may not be installed by a test and thus not loaded yet.
    //drupal_load('module', 'mollom');
    Logger::writeLog();

    module_load_include('inc', 'dblog', 'dblog.admin');

    $this->messages = array();
    $query = db_select('watchdog', 'w')
      ->fields('w')
      ->orderBy('w.timestamp', 'ASC');

    // The comparison logic applied in this function is a bit confusing, since
    // the values of watchdog severity level constants defined by RFC 3164 are
    // negated to their actual "severity level" meaning:
    // RfcLogLevel::EMERGENCY is 0, RfcLogLevel::NOTICE is 5, RfcLogLevel::DEBUG is 7.

    $fail_expected = ($max_severity < RfcLogLevel::NOTICE);
    $had_severe_message = FALSE;
    foreach ($query->execute() as $row) {
      $this->messages[$row->wid] = $row;
      // Only messages with a maximum severity of $max_severity or less severe
      // messages must pass. More severe messages need to fail. See note about
      // severity level constant values above.

      $output = $this->formatMessage($row);
      if ($row->severity >= $max_severity) {
        // Visually separate debug log messages from other messages.
        if ($row->severity == RfcLogLevel::DEBUG) {
          $this->error($output, 'User notice');
        }
        else {
          $this->pass(SafeMarkup::checkPlain($row->type) . ': ' . $output, t('Watchdog'));
        }
      }
      else {
        $this->fail(SafeMarkup::checkPlain($row->type) . ': ' . $output, t('Watchdog'));
      }
      // In case a severe message is expected, non-severe messages always pass,
      // since we would trigger a false positive test failure otherwise.
      // However, in order to actually assert the expectation, there must have
      // been at least one severe log message.
      $had_severe_message = ($had_severe_message || $row->severity < RfcLogLevel::NOTICE);
    }
    // Assert that there was a severe message, in case we expected one.
    if ($fail_expected && !$had_severe_message) {
      $this->fail(t('Severe log message was found.'), t('Watchdog'));
    }
    // Delete processed watchdog messages.
    if (!empty($this->messages)) {
      $seen_ids = array_keys($this->messages);
      db_delete('watchdog')->condition('wid', $seen_ids, 'IN')->execute();
    }
  }

  /**
   * Wraps drupalGet() for additional watchdog message assertion.
   *
   * @param $options
   *   In addition to regular $options that are passed to url():
   *   - watchdog: (optional) Boolean whether to assert that only non-severe
   *     watchdog messages have been logged. Defaults to TRUE. Use FALSE to
   *     negate the watchdog message severity assertion.
   *
   * @see DrupalWebTestCase->drupalGet()
   * @see MollomWebTestCase->assertMollomWatchdogMessages()
   * @see MollomWebTestCase->assertResponseID()
   */
  protected function drupalGet($path, array $options = array(), array $headers = array()) {
    $output = parent::drupalGet($path, $options, $headers);
    $options += array('watchdog' => RfcLogLevel::NOTICE);
    $this->assertMollomWatchdogMessages($options['watchdog']);
    return $output;
  }

  /**
   * Wraps drupalPostForm() for additional watchdog message assertion.
   *
   * @param $options
   *   In addition to regular $options that are passed to url():
   *   - watchdog: (optional) Boolean whether to assert that only non-severe
   *     watchdog messages have been logged. Defaults to TRUE. Use FALSE to
   *     negate the watchdog message severity assertion.
   *
   * @see MollomWebTestCase->assertMollomWatchdogMessages()
   * @see MollomWebTestCase->assertResponseID()
   * @see DrupalWebTestCase->drupalPostForm()
   */
  protected function drupalPostForm($path, $edit, $submit, array $options = array(), array $headers = array(), $form_html_id = NULL, $extra_post = NULL) {
    parent::drupalPostForm($path, $edit, $submit, $options, $headers, $form_html_id, $extra_post);
    $options += array('watchdog' => RfcLogLevel::NOTICE);
    $this->assertMollomWatchdogMessages($options['watchdog']);
  }

  /**
   * Assert that the Mollom session id remains the same.
   *
   * The Mollom session id is only known to one server. If we are communicating
   * with a different Mollom server (due to a refreshed server list or being
   * redirected), then we will get a new session_id.
   *
   * @param $type
   *   The type of ID to assert; e.g., 'contentId', 'captchaId'.
   * @param $id
   *   The ID of $type in the last request, as returned from Mollom.
   * @param $new_expected
   *   (optional) Boolean indicating whether a new ID is expected; e.g., after
   *   incorrectly solving a CAPTCHA.
   */
  protected function assertResponseID($type, $id, $new_expected = FALSE) {
    if (!isset($this->responseIds[$type]) || $new_expected) {
      // Use assertTrue() instead of pass(), to test !empty().
      $this->assertTrue($id, t('New %type: %id', array(
        '%type' => $type,
        '%id' => $id,
      )));
      $this->responseIds[$type] = $id;
    }
    else {
      $this->assertEqual($id, $this->responseIds[$type]);
    }
    return $this->responseIds[$type];
  }

  /**
   * Reset the statically cached Mollom session id.
   *
   * @param $type
   *   The type of ID to reset; e.g., 'contentId', 'captchaId'.
   */
  protected function resetResponseID($type = NULL) {
    if (isset($type)) {
      unset($this->responseIds[$type]);
    }
    else {
      unset($this->responseIds);
    }
  }

  /**
   * Instantiate a Mollom client and make it available on $this->mollom;
   */
  protected function getClient() {
    if (!isset($this->mollom)) {
      // mollom.module may not be enabled in the parent site executing the test.
      //drupal_load('module', 'mollom');
      $this->mollom = \Drupal::service('mollom.client');
    }
    return $this->mollom;
  }

  /**
   * Setup Mollom API keys for testing.
   *
   * New keys are only created if MollomWebTestCase::$createKeys or respectively
   * the 'mollom_testing_create_keys' variable is set to TRUE.
   *
   * @param bool $once
   *   (optional) Whether to disable the 'mollom_testing_create_keys' variable
   *   after the first call (and thus omit API key verifications on every page
   *   request). Defaults to FALSE; i.e., API keys are verified repetitively.
   *
   * Note: This only applies to the Test client or Test Local client.
   *
   * @see MollomWebTestCase::$createKeys
   * @see MollomDrupalTest::__construct()
   * @see MollomDrupalTest::createKeys()
   */
  protected function setKeys($once = FALSE) {
    // Instantiate a Mollom client class.
    // Depending on MollomWebTestCase::$createKeys and ultimately the
    // 'mollom_testing_create_keys' variable, MollomDrupalTest::__construct()
    // will automatically setup testing API keys.
    $this->getClient();

    $this->mollom->createKeys();

    // Make API keys available to test methods.
    if (!empty($this->mollom->publicKey)) {
      $this->publicKey = $this->mollom->publicKey;
      $this->privateKey = $this->mollom->privateKey;

      // Multiple tests might be executed in a single request. Every test sets
      // up a new child site from scratch. The Mollom class with testing API
      // keys still exists in the test, but the configuration is gone.
      $this->mollom->saveKeys();
    }
    if ($once) {
      $this->settings->set('test_mode.create_keys', FALSE)->save();
    }
  }

  /**
   * Calls _mollom_status() directly to verify that current API keys are valid.
   */
  protected function assertValidKeys() {
    $status = MollomUtilities::getAPIKeyStatus(TRUE);
    $this->assertMollomWatchdogMessages();
    $this->assertIdentical($status['isVerified'], TRUE, t('Mollom servers can be contacted and testing API keys are valid.'));
  }

  /**
   * Formats a database log message.
   *
   * This is copied from DbLogController and should be called from there instead.
   *
   * @param object $row
   *   The record from the watchdog table. The object properties are: wid, uid,
   *   severity, type, timestamp, message, variables, link, name.
   *
   * @return string|false
   *   The formatted log message or FALSE if the message or variables properties
   *   are not set.
   */
  public function formatMessage($row) {
    // Check for required properties.
    if (isset($row->message) && isset($row->variables)) {
      // Messages without variables or user specified text.
      if ($row->variables === 'N;') {
        $message = $row->message;
      }
      // Message to translate with injected variables.
      else {
        $message = SafeMarkup::format($row->message, unserialize($row->variables));
      }
    }
    else {
      $message = FALSE;
    }
    return $message;
  }

  /**
   * Wraps drupalGet() for additional watchdog message assertion.
   *
   * @param $options
   *   In addition to regular $options that are passed to url():
   *   - watchdog: (optional) Boolean whether to assert that only non-severe
   *     watchdog messages have been logged. Defaults to TRUE. Use FALSE to
   *     negate the watchdog message severity assertion.
   *
   * @see DrupalWebTestCase->drupalGet()
   * @see MollomWebTestCase->assertMollomWatchdogMessages()
   * @see MollomWebTestCase->assertResponseID()
   */
  /*
  protected function drupalGet($path, array $options = array(), array $headers = array()) {
    $output = parent::drupalGet($path, $options, $headers);
    $options += array('watchdog' => RfcLogLevel::NOTICE);
    $this->assertMollomWatchdogMessages($options['watchdog']);
    return $output;
  }
  */

  /**
   * Wraps drupalPostForm() for additional watchdog message assertion.
   *
   * @param $options
   *   In addition to regular $options that are passed to url():
   *   - watchdog: (optional) Boolean whether to assert that only non-severe
   *     watchdog messages have been logged. Defaults to TRUE. Use FALSE to
   *     negate the watchdog message severity assertion.
   *
   * @see MollomWebTestCase->assertMollomWatchdogMessages()
   * @see MollomWebTestCase->assertResponseID()
   * @see DrupalWebTestCase->drupalPost()
   */
  /*
  protected function drupalPostForm($path, $edit, $submit, array $options = array(), array $headers = array(), $form_html_id = NULL, $extra_post = NULL) {
    $output = parent::drupalPostForm($path, $edit, $submit, $options, $headers, $form_html_id, $extra_post);
    $options += array('watchdog' => RfcLogLevel::NOTICE);
    $this->assertMollomWatchdogMessages($options['watchdog']);
    return $output;
  }
  */
} 
